#!/usr/bin/perl

###↑こちらをサーバーに合わせて変更してください。###

#Copyright (c) 2023 Cocoa
#Released under the MIT license
#https://opensource.org/licenses/mit-license.php

####################

use strict;
use warnings;
use CGI;
use Encode;

my $dir_name = "./diary_data"; #データを入れるディレクトリ
my $pass_ok = ""; #パスワードチェックの結果
my $view_data = ""; #表示するデータ
my $line = ""; #ファイルから読みこんだデータ
my $file_num = 0; #読みこむファイルの番号
my $tag_ok = ""; #aタグとdetailsタグ追加済みのデータを入れる
my $answer = ""; #残りは何文字か（文章での返答）
my $over = ""; #制限文字数を超えたかチェック

#detailsのHTMLタグ
my $details_0 = '<details><summary id="pattern">';
my $details_1 = '</summary><p class="p_details">';
my $details_2 = '</p></details>';

#フォームからデータを受け取る（受け取っていなければ空欄）
my $query = CGI->new;
my $pass = $query->param('pass'); #入力されたパスワード
my $onetime = $query->param('onetime'); #認証済みかチェック
my $log_n = $query->param('log_num'); #どのデータの日記を表示するか
my $submit = $query->param('submit'); #「OK」でパスワードチェック、「記入する」で記入、「確認する」で文字数表示とプレビュー、「編集する」で編集、「設定する」でパスワード設定、「変更する」で日記の設定変更、「削除する」で削除確認、「パスワードの変更」でパスワード変更画面、「パスワードを変更する」で新しいパスワードを保存
my $edit_mode = $query->param('edit_mode'); #「ON」なら編集・削除画面を表示
my $del_ok = $query->param('del_ok'); #「削除する」なら削除実行
my $num_edit = $query->param('num_edit'); #編集・削除する日記の番号
my $word = $query->param('diary'); #記入された日記
my $edit_diary = $query->param('edit_diary'); #編集された日記
#設定変更する場合、フォームからデータを受け取る
my $setting_change = $query->param('setting_change'); #設定変更（アイコン）ボタン（「change」なら日記設定の記入画面）
my $diary_title = $query->param('diary_title'); #日記のタイトル
my $diaries = $query->param('diaries'); #最新日記の表示数
my $home_url = $query->param('home_url'); #HOMEボタンのリンク先
my $home_name = $query->param('home_name'); #HOMEボタンの名前
my $admin_btn = $query->param('admin_btn'); #管理ボタンの有無（0なら非表示、1なら表示）

#現在の日付を取得
my ($second, $minute, $hour, $mday, $month, $year) = localtime;
$month += 1; #monthは0から始まるので+1
$year  += 1900; #yearは1900引かれた数なので+1900

#パスワードが設定されているか
if (-f"${dir_name}/password.txt") {
    #パスワード設定済みの場合
    ($pass_ok,$onetime) = password_check($pass,$onetime,$submit); #パスワードチェック
} elsif ($submit eq '設定する') {
    #パスワードを設定する場合（半角英数字6-8桁かチェック）
    if ($pass =~ /^[a-zA-Z0-9]{6,8}$/) {
        #-dでディレクトリの存在をチェック
        unless (-d"${dir_name}") {
            mkdir("${dir_name}",0700) or die("Error:$!"); #データを保存するディレクトリをパーミッション700で作成
            data_write("password.txt",$pass); #パスワードファイルの作成と書きこみ
            $onetime = &rand_8; #ワンタイムパスワードを作成して、onetime変数に入れる
            $pass_ok = "OK";
        } else {
            $pass_ok = "ERROR"; #パスワードが設定できない（既にディレクトリが存在するため）
        }
    } else {
        $pass_ok = "RETRY"; #設定しようとしたパスワードが6～8桁の半角英数字ではなかった
    }
} else {
    #パスワード未設定の場合
    $pass_ok = "NOT_SET";
}

if ($pass_ok eq 'OK') {
    #パスワード認証後の管理画面
    #HTMLタグ
    my $back_btn = "<p><div class='center'><form action='' method='POST' class='form1'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='戻る' class='btn'>\n</form></div></p>\n"; #編集画面などから戻るボタン
    my $back_btn2 = "<form action='' method='POST' class='form1'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='戻る' class='btn'>\n</form>"; #編集・削除実行後などから戻るボタン
    my $radio_1 = "<label><input type='radio' name='admin_btn' value='1' checked>表示</label><label><input type='radio' name='admin_btn' value='0'>非表示</label>"; #表示にチェック
    my $radio_0 = "<label><input type='radio' name='admin_btn' value='1'>表示</label><label><input type='radio' name='admin_btn' value='0' checked>非表示</label>"; #非表示にチェック
    my $pass_form = "<form action='' method='POST' class='form1'>\n<input type='hidden' name='onetime' value='${onetime}'>\nパス<input type='password' name='pass' size='8' maxlength='8' class='pass_waku' autocomplete='off'><br>\n<input type='submit' name='submit' value='パスワードを変更する' class='btn'></form>"; #パスワード変更画面
    if ($setting_change eq 'change') {
        #設定変更（アイコン）ボタンが押されたら
        #変数の初期値は未設定の場合
        $diary_title = "Diary";
        $diaries = 10;
        $home_url = "";
        $home_name = "";
        $admin_btn = "${radio_1}";
        if (-f("${dir_name}/setting.txt")) {
            my $setting = data_read("setting.txt");
            my @setting_list = split(/\t/, $setting);
            $diary_title = $setting_list[0];
            $diaries = $setting_list[1];
            if ($setting_list[2] eq '{{None}}') {
                #未記入の場合は{{None}}が入っているので削除
                $home_url= "";
            } else {
                $home_url = $setting_list[2];
            }
            if ($setting_list[3] eq '{{None}}') {
                $home_name = "";
            } else {
                $home_name = $setting_list[3];
            }
            if ($setting_list[4] eq '0') {
                #0なら非表示をチェック
                $admin_btn = $radio_0;
            }
        }
        $view_data = "<div class='box2'>\n<h2>日記の設定変更</h2>\n";
        $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<div class='setting_box'><input type='hidden' name='onetime' value='${onetime}'>\n日記のタイトル<br><input type='text' class='waku3' name='diary_title' value='${diary_title}' autocomplete='off'><br>\n最新日記の表示数（1～50件、半角数字）<br><input type='number' class='waku3' name='diaries' value='${diaries}' autocomplete='off'><br>\n";
        $view_data = "${view_data}HOMEボタンのリンク先<br><input type='text' class='waku3' name='home_url' value='${home_url}' placeholder='URL未記入ならボタンは非表示' autocomplete='off'><br>\nHOMEボタンの名前<br><input type='text' class='waku3' name='home_name' value='${home_name}' placeholder='HOME以外に変えるなら記入' autocomplete='off'><br>\n管理ボタン${admin_btn}</div>\n";
        $view_data = "${view_data}<input type='submit' name='submit' value='変更する' class='btn'>&nbsp;<input type='submit' name='submit' value='パスワードの変更' class='btn'>\n</form></div>${back_btn}\n";
    } elsif ($submit eq '変更する') {
        #「変更する」ボタンが押されたら、入力された設定のチェック
        my $setting_error = ""; #タイトル未記入ならEMPTY、URLが正しくない場合はERROR
        if ($diary_title eq '') {
            $setting_error = "EMPTY"; #未記入
        }
        unless ($home_url =~ m{^https?://\S+} || $home_url eq '') {
            $setting_error = "ERROR";
        }
        unless ($diaries >= 1 && $diaries <= 50) {
            $setting_error = "OVER";
        }
        if ($setting_error eq '') {
            $diary_title = tag($diary_title); #タブ文字を削除、タグの無効化
            $diaries = int($diaries);
            if ($home_url eq '') {
                $home_url = '{{None}}';
            } else {
                $home_url =~ s/\s//g; #空白文字があれば削除
                $home_url = tag($home_url); #タブ文字を削除、タグの無効化
            }
            if ($home_name eq '') {
                $home_name = '{{None}}';
            } else {
                $home_name = tag($home_name); #タブ文字を削除、タグの無効化
            }
            my $setting_data = "${diary_title}\t${diaries}\t${home_url}\t${home_name}\t${admin_btn}"; #保存するデータ
            data_write("setting.txt",$setting_data); #データを上書きして設定変更（ファイルがなければ新規作成）
            $view_data = "<div class='box2'>\n<h2>日記の設定を変更しました。</h2>\n${back_btn2}</div>\n";
        } else {
            if ($setting_error eq 'EMPTY') {
                $view_data = "タイトルが未記入です。";
            } elsif ($setting_error eq 'ERROR') {
                $view_data = "リンク先はURLを記入してください。";
            } elsif ($setting_error eq 'OVER') {
                $view_data = "最新日記は1～50件まで表示できます。";
            } else {
                $view_data = "設定できませんでした。";
            }
            my $radio_btn = $radio_1;
            if ($admin_btn eq '0') {
                $radio_btn = $radio_0;
            } 
            $view_data = "<div class='box2'>\n<h2>${view_data}</h2>\n<form action='' method='POST' class='form1'>\n<div class='setting_box'><input type='hidden' name='onetime' value='${onetime}'>\n日記のタイトル<br><input type='text' class='waku3' name='diary_title' value='${diary_title}' autocomplete='off'><br>\n最新日記の表示数（1～50件、半角数字）<br><input type='number' class='waku3' name='diaries' value='${diaries}' autocomplete='off'><br>\n";
            $view_data = "${view_data}HOMEボタンのリンク先<br><input type='text' class='waku3' name='home_url' value='${home_url}' placeholder='URL未記入ならボタンは非表示' autocomplete='off'><br>\nHOMEボタンの名前<br><input type='text' class='waku3' name='home_name' value='${home_name}' placeholder='HOME以外に変えるなら記入' autocomplete='off'><br>\n管理ボタン${radio_btn}</div>\n<input type='submit' name='submit' value='変更する' class='btn'>&nbsp;<input type='submit' name='submit' value='パスワードの変更' class='btn'>\n</form></div>${back_btn}\n";
        }
    } elsif ($submit eq 'パスワードの変更') {
        #新しいパスワードの入力画面
        $view_data = "<div class='box2'>\n<h2>新しいパスワードを入力してください。</h2>\n${pass_form}</div>\n${back_btn}";
    } elsif ($submit eq 'パスワードを変更する') {
        #新しいパスワードが入力された
        if ($pass =~ /^[a-zA-Z0-9]{6,8}$/) {
            data_write("password.txt",$pass); #新しいパスワードを上書き保存
            $view_data = "<div class='box2'>\n<h2>パスワードが変更されました。</h2>\n${back_btn2}</div>\n";
        } else {
            #変更しようとしたパスワードが6～8桁の半角英数字ではなかった
            $view_data = "<div class='box2'>\n<h2>6～8桁の半角英数字で記入してください。</h2>\n${pass_form}</div>\n${back_btn}";
        }
    } elsif ($num_edit eq '') {
        #編集・削除する日記の番号が指定されていない
        my $write_btn = "<br>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='記入する' class='btn'><input type='submit' name='submit' value='確認する' class='btn'>"; #記入ボタンと確認ボタン
        if ($submit eq '記入する' || $submit eq '確認する') {
            #日記を記入する場合
            ($answer,$over) = word_count($word); #残りの文字数をチェック
            $word = tag($word); #タブ文字があったらなくす、タグの無効化
            if ($word eq '') {
                $view_data = "<div class='box2'>\n<h2>日記を記入してください。</h2>\n";
                $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='diary' rows='10' class='waku2' placeholder='（500文字まで）' autocomplete='off'></textarea>${write_btn}</form></div>\n";
            } elsif ($submit eq '確認する') {
                my $preview = preview_diary($word,$details_0,$details_1,$details_2); #日記のプレビュー
                $view_data = "<div class='box2'>\n<h2>${answer}</h2>\n";
                $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='diary' rows='10' class='waku2' autocomplete='off'>${word}</textarea>${write_btn}</form></div>\n${preview}";
            } elsif ($over eq 'OVER') {
                $view_data = "<div class='box2'>\n<h2>${answer}</h2>\n";
                $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='diary' rows='10' class='waku2' autocomplete='off'>${word}</textarea>${write_btn}</form></div>\n";
            } else {
                $word = change_data($word); #日記データを変換
                my $num = "";
                $file_num = 0;

                #ファイルが存在するかチェック（なければ作る）
                if (-f"${dir_name}/data0.txt") {
                    while (-f"${dir_name}/data${file_num}.txt"){
                        $file_num += 1;
                        }
                    $file_num -= 1;
                } else {
                    open(my $fh, ">", "${dir_name}/data0.txt") or die("Error:$!");
                    close($fh);
                }

                $line = data_read("data${file_num}.txt"); #ファイルの読みこみ

                #もしデータがなかった場合
                if ($line eq ''){
                    $num = "1";
                } elsif ($line =~ /\d+/){
                    #ファイルの最初に書かれた数字（最後に書きこまれた数字）をチェック
                    $num = $& + 1; #$&はマッチした文字列、文字列を数値に変換しなくてもperlは計算できる
                }

                #データ100個ごとにファイルを分ける
                $file_num = int($num/100);
                #ファイルがなければ新しく作るので、データを空にする（番号取得のために前のファイルのデータが読みこまれているので）
                unless (-f"${dir_name}/data${file_num}.txt") {
                    $line = "";
                }

                my $write_data = sprintf("$num\t%04d/%02d/%02d\t%02d:%02d\t${word}\t\t\t$line",${year},${month},${mday},${hour},${minute});

                data_write("data${file_num}.txt",$write_data); #ファイルに上書きで書きこみ（ファイルがなければ新規作成）
                
                $tag_ok = tag_add($word,$details_0,$details_1,$details_2); #日記データにHTMLタグを追加
                $view_data = "<div class='box2'>\n<h2>日記を記入しました。</h2>\n${tag_ok}</div>\n";
            }
            $view_data = "${view_data}${back_btn}";
        } else {
            #日記一覧を表示
            $file_num = 0; #初期化
            #データのファイルが存在するかチェック
            if (-f"${dir_name}/data${file_num}.txt") {
                while (-f"${dir_name}/data${file_num}.txt"){
                    $file_num += 1;
                }
                $file_num -= 1;
                my $n = $file_num; #プルダウンメニュー用
                my $new_btn = "";
                my $what_diary = "新しい日記"; #日記一覧のタイトル

                #どのログを見るか選んでいたら
                if ($log_n ne '') {
                    $file_num = $log_n;
                    $new_btn = "<form action='' method='POST'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='最新日記' class='btn'></form>\n"; #最新日記のボタンを追加
                    $what_diary = "これまでの日記";
                }

                $line = data_read("data${file_num}.txt"); #ファイルの読みこみ
                my $count_view = 0; #いくつ日記を表示したかカウントする
                #表示する日記の件数をチェック
                if (-f("${dir_name}/setting.txt")) {
                    my $setting = data_read("setting.txt");
                    my @setting_list = split(/\t/, $setting);
                    $diaries = $setting_list[1];
                } else {
                    $diaries = 10; #未設定の場合
                }

                ($view_data,$count_view) = view_diary($view_data,$count_view,$line,$log_n,$diaries,$details_0,$details_1,$details_2); #表示する日記

                #ファイル読みこみ2回目（最新日記で、カウント上限に達していない場合）
                if ($file_num > 0 && $count_view < $diaries && $log_n eq '') {
                    $file_num -= 1;
                    $line = data_read("data${file_num}.txt"); #ファイルの読みこみ
                    ($view_data,$count_view) = view_diary($view_data,$count_view,$line,$log_n,$diaries,$details_0,$details_1,$details_2); #表示する日記
                }
                
                #プルダウンメニュー
                my $btn = "";
                my $n_100 = 0;
                while ($n >= 1) {
                    $n_100 = $n*100;
                    $btn = "${btn}<option value='${n}'>No.${n_100}-</option>\n";
                    $n -= 1;
                }
                $view_data = "<div class='center'><h1>${what_diary}</h1></div>\n<div class='center'>${new_btn}<form action='' method='POST'>\n<select name='log_num'>${btn}<option value='0'>No.1-</option>\n</select>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='見る' class='btn'>\n</form></div>\n${view_data}";
            } else {
                $view_data = "<div class='center'>データが存在しません。</div>\n";
            }
            $view_data = "<div class='center'><h1>日記の記入</h1>\n<form action='' method='POST'>\n<textarea name='diary' rows='5' class='waku' placeholder='（500文字まで）' autocomplete='off'></textarea>${write_btn}</form>\n<p><form action='' method='POST'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='hidden' name='setting_change' value='change'>\n<input type='submit' name='submit' value='&#xe8b8;' class='btn_icon'>\n</form><a href='index.cgi' class='btn_icon'>&#xea19;</a></p>\n</div>\n${view_data}";
        }
    } else {
        #データを編集・削除する場合
        my $edit_btn = "\n<br><input type='hidden' name='num_edit' value='${num_edit}'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='submit' value='編集する' class='btn'>\n<input type='submit' name='submit' value='確認する' class='btn'>\n&nbsp;&nbsp;<input type='submit' name='submit' value='削除する' class='btn2'>\n"; #編集・確認・削除ボタン
        if ($edit_mode eq 'ON') {
            #日記一覧の小さな編集ボタンが押された場合
            my $file_num = int($num_edit/100);
            $line = data_read("data${file_num}.txt"); #ファイルの読みこみ
            my @list = split(/\t\t\t/, $line);
            foreach my $parts(@list){
                my @list2 = split(/\t/, $parts);
                if ($list2[0] eq $num_edit && $list2[1] ne "0") {
                    $edit_diary = $list2[3];
                }
            }
            $edit_diary =~ s/<br>/\n/g; #日記データを編集できる形に戻す
            $view_data = "<div class='box2'>\n<h2>No.${num_edit}の編集・削除</h2>\n";
            $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='edit_diary' rows='10' class='waku2' autocomplete='off'>${edit_diary}</textarea>${edit_btn}</form></div>\n${back_btn}";
        } elsif ($submit eq '確認する') {
            ($answer,$over) = word_count($edit_diary); #残りの文字数をチェック
            $edit_diary = tag($edit_diary); #タブ文字があったらなくす、タグの無効化
            my $preview = preview_diary($edit_diary,$details_0,$details_1,$details_2); #日記のプレビュー
            $view_data = "<div class='box2'>\n<h2>${answer}</h2>\n";
            $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='edit_diary' rows='10' class='waku2' autocomplete='off'>${edit_diary}</textarea>${edit_btn}</form></div>\n${preview}${back_btn}";
        } elsif ($submit eq '編集する' && $edit_diary eq '') {
            #編集するボタンが押されたが、何も書かれていない場合
            $view_data = "<div class='box2'>\n<h2>日記が記入されていません。</h2>\n";
            $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='edit_diary' rows='10' class='waku2' autocomplete='off'></textarea>${edit_btn}</form></div>\n${back_btn}";
        } elsif ($submit eq '編集する') {
            #編集実行
            ($answer,$over) = word_count($edit_diary); #残りの文字数をチェック
            $edit_diary = tag($edit_diary); #タブ文字があったらなくす、タグの無効化
            if ($over eq 'OVER') {
                $view_data = "<div class='box2'>\n<h2>${answer}</h2>\n";
                $view_data = "${view_data}<form action='' method='POST' class='form1'>\n<textarea name='edit_diary' rows='10' class='waku2' autocomplete='off'>${edit_diary}</textarea>${edit_btn}</form></div>\n${back_btn}";
            } else {
                ($view_data,$edit_diary) = edit_delete($edit_diary,$num_edit,$submit,$del_ok); #編集または削除の実行
                $tag_ok = tag_add($edit_diary,$details_0,$details_1,$details_2); #日記データにHTMLタグを追加
                $view_data = "<div class='box2'>\n<h2>${view_data}</h2>\n${tag_ok}\n${back_btn2}</div>\n";
            }
        } elsif ($submit eq '削除する') {
            #削除確認
            $view_data = "<div class='box2'>\n<h2>No.${num_edit}を本当に削除しますか？</h2>\n";
            $view_data = "${view_data}\n<form action='' method='POST' class='form1'>\n<input type='hidden' name='num_edit' value='${num_edit}'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='submit' name='del_ok' value='削除する' class='btn2'>\n</form></div>\n${back_btn}";
        } elsif ($del_ok eq '削除する') {
            #削除実行
            ($view_data,$edit_diary) = edit_delete($edit_diary,$num_edit,$submit,$del_ok); #編集または削除の実行
            $view_data = "<div class='box2'>\n<h2>${view_data}</h2>\n${back_btn2}</div>\n";
        } else {
            $view_data = "日記を編集できませんでした。";
            $view_data = "<div class='box2'>\n<h2>${view_data}</h2>\n${back_btn2}</div>\n";
        }
    }
} elsif ($pass_ok eq "NOT_SET" || $pass_ok eq 'RETRY') {
    #パスワード設定前
    my $password_message0 = "パスワードの設定（半角英数字6～8桁）";
    if ($pass_ok eq 'RETRY') {
        $password_message0 = "6～8桁の半角英数字で記入してください。";
    }
    $view_data = "<div class='box2'>\n<h2>${password_message0}</h2>\n<form action='' method='POST' class='form1'>\nパス<input type='password' name='pass' size='8' maxlength='8' class='pass_waku' autocomplete='off'><br>\n<input type='submit' name='submit' value='設定する' class='btn'></form></div><p><div class='center'><a href='index.cgi' class='btn_icon'>&#xea19;</a></div></p>";
} else {
    #パスワード入力前、パスワードが間違っていた場合など
    my $password_message = "パスワード入力"; #pass_okが空欄（パスワード入力前）
    if ($pass_ok eq 'NG') {
        $password_message = "パスワードが違います。";
    } elsif ($pass_ok eq 'EMPTY') {
        $password_message = "パスワードが入力されていません。";
    } elsif ($pass_ok eq 'ERROR') {
        $password_message = "パスワードが設定できませんでした。";
    }
    #パスワード入力画面
    $view_data = "<div class='box2'>\n<h2>${password_message}</h2>\n<form action='' method='POST' class='form1'>\nパス<input type='password' name='pass' size='8' maxlength='8' class='pass_waku' autocomplete='off'><br>\n<input type='submit' name='submit' value='OK' class='btn'></form></div><p><div class='center'><a href='index.cgi' class='btn_icon'>&#xea19;</a></div></p>";
}

###サブルーチン

#パスワードチェック
sub password_check {
    my ($pass,$onetime,$submit) = @_;
    my $pass_ok = ""; #パスワード入力前なら空欄

    if ($submit eq 'OK') {
        #パスワードが入力された（ボタンが押された）場合
        my $my_pass = data_read("password.txt"); #保存されたパスワードの読みこみ
        if ($pass eq '') {
            #パスワード未入力
            $pass_ok = "EMPTY";
        } elsif ($pass eq $my_pass) {
            #記入されたパスワードが正しい場合
            $pass_ok = "OK";
            $onetime = &rand_8; #ワンタイムパスワードを作成して更新し、onetime変数に入れる
        } else {
            #パスワードが間違っている
            $pass_ok = "NG";
        }
    } else {
        #パスワード認証済みか、パスワード入力前の場合
        if ($onetime ne '' && -f"${dir_name}/onetime_password.txt") {
            #ワンタイムパスワードを所持していて、ワンタイムパスワードのファイルがあるなら
            my $onetime_pass = data_read("onetime_password.txt"); #保存されたワンタイムパスワードの読みこみ
            if ($onetime_pass eq $onetime) {
                #パスワード認証済みの場合
                $pass_ok = "OK";
            }
        }
        #パスワード入力前なら、$pass_okは空欄のまま
    }
    return ($pass_ok,$onetime);
}

#残りの文字数をチェック
sub word_count {
    my ($word) = @_;
    my $count = length(decode('utf-8', $word));
    $count = 500 - $count; #残りの文字数を計算
    my $over = ""; #文字数を超えていなければ空欄
    my $answer = ""; #返答
    if ($count < 0) {
        $over = "OVER";
        $count = $count*-1;
        $answer = "制限文字数を${count}文字超えています。";
    } else {
        $answer = "残り${count}文字、記入できます。";
    }
    return ($answer,$over);
}

#タグの無効化（エスケープ処理）、タブ文字があったらなくす
sub tag {
    my ($word) = @_;
    $word =~ s/\t//g;
    $word =~ s/&/&amp;/g; #「&」は先に置換（あとからだと↓の&まで変わってしまうので）
    $word =~ s/\#/&#35;/g;
    $word =~ s/</&lt;/g;
    $word =~ s/>/&gt;/g;
    $word =~ s/\'/&apos;/g;
    $word =~ s/\"/&quot;/g;
    $word =~ s/\(/&#40;/g;
    $word =~ s/\)/&#41;/g;
    $word =~ s/\?/&quest;/g;
    $word =~ s/\*/&ast;/g;
    $word =~ s/\$/&#x24;/g;
    $word =~ s/\^/&#x5e;/g;
    $word =~ s/ /&nbsp;/g;
    return $word;
}

#記入された日記データを変換
sub change_data {
    my ($word) = @_; #引数を受け取る
    $word =~ s/\r\n/\n/g;
    #改行はbrタグに変換（一行のデータにするため、改行をなくす）
    $word =~ s/\n/<br>/g; #<と>が変わらないように、タグの無効化後に置換
    $word =~ s/\r//g;
    return $word;
}

#表示する日記
sub view_diary {
    my ($view_data,$count_view,$line,$log_n,$diaries,$details_0,$details_1,$details_2) = @_;
    my @list = split(/\t\t\t/, $line);
    foreach my $parts(@list){
        my @list2 = split(/\t/, $parts);
        #削除されたデータ以外を表示
        if ($list2[1] ne "0" && $count_view < $diaries){
            $tag_ok = tag_add($list2[3],$details_0,$details_1,$details_2); #日記データにHTMLタグを追加
            $view_data = "${view_data}<div class='box'>${tag_ok}\n<div class='diary-date'><form action='' method='POST' class='form1'>$list2[1]&#91;$list2[2]&#93;No.$list2[0]\n";
            $view_data = "${view_data}<input type='hidden' name='num_edit' value='$list2[0]'>\n<input type='hidden' name='onetime' value='${onetime}'>\n<input type='hidden' name='edit_mode' value='ON'>\n<input value='&#xf88d;' type='submit' name='submit' class='btn_edit'>\n</form></div></div>\n";
            #新しい日記の場合、設定された件数まで日記を表示
            if ($log_n eq '') {
                $count_view += 1;
            }
        }
    }
    return ($view_data,$count_view);
}

#日記表示前にaタグとdetailsタグをつける
sub tag_add {
    my ($data,$details_0,$details_1,$details_2) = @_;
    $data =~ s/<br>/\n/g; #検索用
    $data = "${data}\n"; #検索用

    #「」URLにリンクを貼る
    my @urls = $data =~ /(「.+?」https?:\/\/[\-_\.!~\'\(\)a-zA-Z0-9;\/:@&=,%#\?\*]+\n)/g;
    foreach (@urls){
        my $word_url = "$_";
        chomp($word_url);
        $word_url =~ s/.*「/「/;
        my $url_link = $word_url;
        my $url_what = $word_url;
        $url_link =~ s/「.+?」//;
        $url_what =~ s/https?:\/\/.+//;
        $url_what =~ s/「//;
        $url_what =~ s/」//;
        $data =~ s/${word_url}\n/<a href=\"${url_link}\" target=\"_blank\" rel=\"noopener noreferrer\">${url_what}<\/a>\n/;
    }
    
    #URLのみの場合もリンクを貼る
    my @urls_2 = $data =~ /(https?:\/\/[\-_\.!~\'\(\)a-zA-Z0-9;\/:@&=,%#\?]+)/g;
    @urls_2 = sort { length $b <=> length $a } @urls_2;
    foreach (@urls_2){
        my $word_url2 = "$_";
        $data =~ s/${word_url2}\n/<a href=\"${word_url2}\" target=\"_blank\" rel=\"noopener noreferrer\">${word_url2}<\/a>\n/;
    }

    #折りたたむ文章があれば、detailsタグをつける
    my @details = $data =~ /(【.+?：[\S\s]+?】)/g;
    foreach (@details){
        my $word_details = "$_";
        $word_details =~ s/【//;
        $word_details =~ s/】//;
        my $details_title = $word_details; #続きを読むボタン
        my $details_hide = $word_details; #隠す文章
        $details_title =~ s/：.+//s; #ボタンの文字を抽出
        $details_hide =~ s/^.+?：\s?//; #隠す文章を取り出す
        $data =~ s/【${word_details}】\n/${details_0}${details_title}${details_1}${details_hide}${details_2}/;
    }
    chomp($data); #検索用の末尾の改行文字を削除
    $data =~ s/\n/<br>/g; #brタグを元に戻す
    return $data;
}

#日記のプレビュー
sub preview_diary {
    my ($data,$details_0,$details_1,$details_2) = @_;
    my $preview = change_data($data); #日記データを変換
    $preview = tag_add($preview,$details_0,$details_1,$details_2); #HTMLタグを追加
    $preview = "<div class='box'>${preview}\n<div class='diary-date'>Preview</div></div>\n";
    return $preview;
}

#編集・削除の実行
sub edit_delete {
    my ($edit_diary,$num_edit,$submit,$del_ok) = @_;
    my $result = ""; #結果を入れる変数
    $edit_diary = change_data($edit_diary); #日記データを変換
    my $file_num = int($num_edit/100);
    #データのファイルが存在するかチェック
    if (-f"${dir_name}/data${file_num}.txt") {
        my $line = data_read("data${file_num}.txt"); #ファイルの読みこみ
        my @list = split(/\t\t\t/, $line);

        my $edit_completed = ""; #編集・削除が完了したか確認するための変数
        my $line_new = ""; #新しいデータ
        foreach my $parts(@list){
            my @list2 = split(/\t/, $parts);
            if ($list2[0] eq $num_edit && $list2[1] ne "0") {
                #list2[0]は番号、list2[1]は日付、list2[2]は時刻、list2[3]は日記
                if ($submit eq '編集する') {
                    $line_new = "${line_new}$list2[0]\t$list2[1]\t$list2[2]\t${edit_diary}\t\t\t";
                    $edit_completed = "ok"; #編集完了
                } elsif ($del_ok eq '削除する') {
                    $line_new = "${line_new}$list2[0]\t0\t0\t0\t\t\t";
                    $edit_completed = "ok"; #削除（編集）完了
                }
            } else {
                $line_new = "${line_new}$list2[0]\t$list2[1]\t$list2[2]\t$list2[3]\t\t\t";
            }
        }

        if ($edit_completed eq 'ok') {
            data_write("data${file_num}.txt",$line_new); #ファイルに上書きで書きこみ
            if ($submit eq '編集する') {
                $result = "No.${num_edit}を編集しました。";
            } elsif ($del_ok eq '削除する') {
                $result = "No.${num_edit}を削除しました。";
            }
        } else {
            $result = "No.${num_edit}は存在しません。";
        }
    } else {
        $result = "No.${num_edit}は存在しません。";
    }
    return ($result,$edit_diary);
}

#8桁のランダム文字列を作成して、ワンタイムパスワードを更新
sub rand_8 {
    #この文字を使う↓
    my @word_list = (); #配列を作成
    push @word_list,('a'..'z'); #pushは末尾に追加
    push @word_list,('A'..'Z');
    push @word_list,('0'..'9');

    my $rand_word = ""; #作成したランダムな文字列を入れる変数

    for (my $i=0; $i<8; $i++){
        $rand_word .= @word_list[int(rand($#word_list+1))];
    }
    data_write("onetime_password.txt",$rand_word); #ワンタイムパスワードを新しいものに更新（ファイルに上書きで書きこみ、ファイルがなければ作られる）
    return $rand_word;
}

#ファイルの読みこみ（最初の行のみ）
sub data_read {
    my ($file_name) = @_; #引数を受け取る
    open(DATAFILE, "< ${dir_name}/${file_name}") or die("Error:$!");
    my $line = <DATAFILE>;
    close(DATAFILE);
    return $line;
}

#ファイルの上書き（ファイルがなければ新規作成）
sub data_write {
    my ($file_name,$w_data) = @_; #引数を受け取る
    #ファイルに上書きで書きこみ「>」
    open (DATAFILE,"> ${dir_name}/${file_name}") or die("Error:$!");
    print DATAFILE "$w_data";
    close(DATAFILE);
    chmod 0600, "${dir_name}/${file_name}" or die("Error:$!"); #ファイルのパーミッションを600に変更
}

###表示するデータ

print "Content-type: text/html; charset=utf-8\n\n";

print "<html lang='ja'>\n<head>\n<meta charset='utf-8'>\n<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n<meta name='robots' content='noindex,nofollow,noarchive,noimageindex'>\n";
print "<title>日記管理画面</title>\n<link rel='stylesheet' href='diary.css'>\n<link rel='stylesheet' href='https://fonts.googleapis.com/css2?family=Material+Symbols+Outlined'>\n</head>\n<body>\n${view_data}</body>\n</html>";