#!/usr/bin/perl

###↑こちらをサーバーに合わせて変更してください。###

#Copyright (c) 2023 Cocoa
#Released under the MIT license
#https://opensource.org/licenses/mit-license.php

####################

use strict;
use warnings;
use CGI;

my $line = "";
my $view_data = "";
my $file_num = 0;
my $select_box = ""; #セレクトボックス
my $btn_all = ""; #ボタンすべて（最新日記、HOME、管理）
my $template = ""; #ページ全体のテンプレート
my $diary_template = ""; #日記表示部分のテンプレート
my $template_file = "./template.html"; #テンプレートファイル
my $dir_name = "./diary_data"; #日記データの入ったディレクトリ
my $tag_ok = ""; #aタグとdetailsタグ追加済みのデータを入れる変数
my $home_btn = ""; #HOMEボタン

#フォームからデータを受け取る
my $query = CGI->new;
my $log_n = $query->param('log_num');

#日記設定の読みこみ
#初期値は未設定の場合
my $diary_title = "Diary"; #日記のタイトル
my $diaries = 10; #最新日記の表示数
my $home_url = ""; #HOMEボタンのリンク先
my $home_name = ""; #HOMEボタンの名前
my $admin_btn = "<a href='diary_edit.cgi' class='btn'>管理</a>"; #管理ボタン
if (-f("${dir_name}/setting.txt")) {
    my $setting = data_read("${dir_name}/setting.txt");
    my @setting_list = split(/\t/, $setting);
    $diary_title = $setting_list[0];
    $diaries = $setting_list[1];
    if ($setting_list[2] eq '{{None}}') {
        #未記入の場合は{{None}}が入っているので削除
        $home_url = "";
    } else {
        $home_url = $setting_list[2];
    }
    if ($setting_list[3] eq '{{None}}') {
        $home_name = "";
    } else {
        $home_name = $setting_list[3];
    }
    if ($setting_list[4] eq '0') {
        $admin_btn = ""; #（1ならボタン表示、0ならボタン非表示）
    }
}

#URLが設定されている場合（HOMEボタンを表示）
if ($home_url ne '') {
    if ($home_name ne ''){
        #HOMEボタンの名前が設定されている場合
        $home_btn = "<a href='${home_url}' class='btn'>${home_name}</a>";
    } else {
        $home_btn = "<a href='${home_url}' class='btn'>HOME</a>";
    }
}

#テンプレートの読みこみ（変数の初期値はテンプレートがない場合、管理画面と同じデザイン）
my $diary_template = "<div class='box'>{{diary_data}}\n<div class='diary-date'>{{diary_day}}[{{diary_time}}]No.{{diary_number}}\n</div></div>";
my $details_0 = '<details><summary id="pattern">';
my $details_1 = '</summary><p class="p_details">';
my $details_2 = '</p></details>';

if (-f"$template_file") {
    #テンプレートがあれば
    $template = data_read_all("$template_file"); #テンプレートファイルの読みこみ
    $template =~ s/\r\n/\n/g;
    ($details_0,$details_1,$details_2) = details_template_check($template,$details_0,$details_1,$details_2); #detailsのテンプレートの読みこみ

    #日記表示部分のテンプレートを抜き出す
    $template =~ s/\{\{details_start\}\}.+\{\{details_end\}\}\s?//s; #先にdetails部分を削除
    if ($template =~ /\{\{diary_start\}\}.+\{\{diary_end\}\}/s){ #正規表現のオプションのs（右のs）は「.」（改行を除くすべての文字）に改行もマッチさせる
        $diary_template = $&; #$&はマッチした文字列
        $diary_template =~ s/\{\{diary_start\}\}\s?//;
        $diary_template =~ s/\{\{diary_end\}\}//;
    }
    chomp($diary_template)
}

#データのファイルが存在するかチェック
if (-f"${dir_name}/data${file_num}.txt") {
    while (-f"${dir_name}/data${file_num}.txt"){
        $file_num += 1;
    }
    $file_num -= 1;
    my $n = $file_num; #プルダウンメニュー用
    my $new_btn = "";

    #どのログを見るか選んでいたら
    if ($log_n ne '') {
        $file_num = $log_n;
        $new_btn = "<a href='index.cgi' class='btn'>最新</a>"; #最新日記のボタンを追加
    }

    $line = data_read("${dir_name}/data${file_num}.txt"); #ファイルの読みこみ
    my $count_view = 0; #いくつ日記を表示したかカウントする

    ($view_data,$count_view) = view_diary($view_data,$count_view,$line,$log_n,$diary_template,$diaries,$details_0,$details_1,$details_2); #表示する日記
    
    #ファイル読みこみ2回目
    if ($file_num > 0 && $count_view < $diaries && $log_n eq '') {
        $file_num -= 1;
        $line = data_read("${dir_name}/data${file_num}.txt"); #ファイルの読みこみ
        ($view_data,$count_view) = view_diary($view_data,$count_view,$line,$log_n,$diary_template,$diaries,$details_0,$details_1,$details_2); #表示する日記
    }

    #プルダウンメニュー
    my $n_100 = 0;
    while ($n >= 1) {
        $n_100 = $n*100;
        $select_box = "${select_box}<option value='${n}'>No.${n_100}-</option>\n";
        $n -= 1;
    }
    $select_box = "<form action='' method='POST'>\n<select name='log_num'>${select_box}<option value='0'>No.1-</option>\n</select>\n<input type='submit' name='submit' value='読む' class='btn'></form>\n";
    $btn_all = "${new_btn}";
} else {
    $select_box = "<p>データが存在しません。</p>\n";
}

$btn_all = "${btn_all}${home_btn}${admin_btn}\n";

#details部分のテンプレートを分割して変数に入れる
sub details_template_check {
    my ($template,$details_template_0,$details_template_1,$details_template_2) = @_; #引数を受け取る
    my $details_template_all = ""; #details部分のテンプレート
    if ($template =~ /\{\{details_start\}\}.+\{\{details_end\}\}/s){
        $details_template_all = $&;
        $details_template_all =~ s/\n//g; #改行があれば削除
        $details_template_all =~ s/\r//g; #改行があれば削除
        $details_template_all =~ s/\t//g; #タブがあれば削除
        $details_template_all =~ s/\{\{details_start\}\}//;
        $details_template_all =~ s/\{\{details_end\}\}//;
        $details_template_0 = $details_template_all;
        $details_template_0 =~ s/\{\{details_summary\}\}.+//s;
        $details_template_1 = $details_template_all;
        $details_template_1 =~ s/${details_template_0}\{\{details_summary\}\}//;
        $details_template_1 =~ s/\{\{details_hide\}\}.+//s;
        $details_template_2 = $details_template_all;
        $details_template_2 =~ s/.+\{\{details_hide\}\}//s;
    }
    return ($details_template_0,$details_template_1,$details_template_2);
}

#日記表示前にaタグとdetailsタグをつける
sub tag_add {
    my ($data,$details_0,$details_1,$details_2) = @_; #引数を受け取る
    $data =~ s/<br>/\n/g; #検索用
    $data = "${data}\n"; #検索用

    #「」URLにリンクを貼る
    my @urls = $data =~ /(「.+?」https?:\/\/[\-_\.!~\'\(\)a-zA-Z0-9;\/:@&=,%#\?\*]+\n)/g; #+?で最短一致（\n込みで検索すると最短一致にならない？）
    foreach (@urls){
        my $word_url = "$_";
        chomp($word_url); #末尾の改行文字があれば削除
        $word_url =~ s/.*「/「/; #「より左の文字を削除
        my $url_link = $word_url;
        my $url_what = $word_url;
        $url_link =~ s/「.+?」//;
        $url_what =~ s/https?:\/\/.+//;
        $url_what =~ s/「//;
        $url_what =~ s/」//;
        $data =~ s/${word_url}\n/<a href=\"${url_link}\" target=\"_blank\" rel=\"noopener noreferrer\">${url_what}<\/a>\n/;
    }
    
    #URLのみの場合もリンクを貼る
    my @urls_2 = $data =~ /(https?:\/\/[\-_\.!~\'\(\)a-zA-Z0-9;\/:@&=,%#\?]+)/g;
    @urls_2 = sort { length $b <=> length $a } @urls_2;
    foreach (@urls_2){
        my $word_url2 = "$_";
        $data =~ s/${word_url2}\n/<a href=\"${word_url2}\" target=\"_blank\" rel=\"noopener noreferrer\">${word_url2}<\/a>\n/;
    }

    #折りたたむ文章があれば、detailsタグをつける
    my @details = $data =~ /(【.+?：[\S\s]+?】)/g; #\sは空白文字（改行含む）、\Sは空白以外の文字、+?で最短一致、+だけだと1文字以上一致（最長一致）、?だけだと0文字か1文字一致
    foreach (@details){
        my $word_details = "$_";
        $word_details =~ s/【//;
        $word_details =~ s/】//;
        my $details_title = $word_details; #続きを読むボタン
        my $details_hide = $word_details; #隠す文章
        $details_title =~ s/：.+//s; #ボタンの文字を抽出
        $details_hide =~ s/^.+?：\s?//; #隠す文章を取り出す
        $data =~ s/【${word_details}】\n/${details_0}${details_title}${details_1}${details_hide}${details_2}/;
    }
    chomp($data); #検索用の末尾の改行文字を削除
    $data =~ s/\n/<br>/g; #brタグを元に戻す
    return $data; #戻り値
}

#ファイルの読みこみ（一行のみ）
sub data_read {
    my ($file_name) = @_; #引数を受け取る
    open(DATAFILE, "< ${file_name}") or die("Error:$!");
    my $line = <DATAFILE>;
    close(DATAFILE);
    return $line; #戻り値
}

#ファイルの読みこみ（すべての行）
sub data_read_all {
    my ($file_name) = @_; #引数を受け取る
    open(DATAFILE, "< ${file_name}") or die("Error:$!");
    my $line_all;
    while(my $line = <DATAFILE>){
        chomp($line);
        $line_all = "${line_all}${line}\n";
        }
    close(DATAFILE);
    return $line_all; #戻り値
}

#表示する日記
sub view_diary {
    my ($view_data,$count_view,$line,$log_n,$diary_template,$diaries,$details_0,$details_1,$details_2) = @_; #引数を受け取る
    my @list = split(/\t\t\t/, $line);
    foreach my $parts(@list){
        my @list2 = split(/\t/, $parts);
        #削除されたデータ以外を表示
        if ($list2[1] ne "0" && $count_view < $diaries){
            $tag_ok = tag_add($list2[3],$details_0,$details_1,$details_2); #日記データにaタグとdetailsタグを追加
            my $diary_oneday = diary_parts($tag_ok,$list2[1],$list2[2],$list2[0],$diary_template); #日記ひとつ分のデータ
            $view_data = "${view_data}${diary_oneday}";
            #最新日記は、指定された件数まで日記を表示
            if ($log_n eq '') {
                $count_view += 1;
            }
            }
        }
    return ($view_data,$count_view);
}

#日記ひとつ分のデータ
sub diary_parts {
    my ($diary_data,$diary_day,$diary_time,$diary_number,$diary_template) = @_; #引数を受け取る
    $diary_template =~ s/\{\{diary_data\}\}/${diary_data}/g;
    $diary_template =~ s/\{\{diary_day\}\}/${diary_day}/g;
    $diary_template =~ s/\{\{diary_time\}\}/${diary_time}/g;
    $diary_template =~ s/\{\{diary_number\}\}/${diary_number}/g;
    return $diary_template;
}

###表示するデータ

if (-f"$template_file") {
    #テンプレートがあれば
    $template =~ s/\{\{diary_start\}\}.+\{\{diary_end\}\}/${view_data}/s;
    $template =~ s/\{\{diary_title\}\}/${diary_title}/g;
    $template =~ s/\{\{select_box\}\}\s?/${select_box}/; #\sは空白文字（改行含む）、?は0文字か1文字
    $template =~ s/\{\{btn\}\}\s?/${btn_all}/;
} else {
    #テンプレートがなければ、管理画面と同じデザイン
    $template = "<html lang='ja'>\n<head>\n<meta charset='utf-8'>\n<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n";
    $template = "${template}<title>${diary_title}</title>\n<link rel='stylesheet' href='diary.css'>\n</head>\n<body>\n<div class='center'><h1>${diary_title}</h1></div>\n${view_data}<div class='center'>${select_box}${btn_all}</div>\n</body>\n</html>";
}

print "Content-type: text/html; charset=utf-8\n\n";
print "${template}";